; OLD VERSION -- USE EXTRAS.AIH INSTEAD

; Low-level asm routines
;
; ** 8-bit routines:
; brelop : compare byte
;
; ** 16-bit routines:
; wrelop : compare word
; amul : signed multiply
; lmul : unsigned multiply
; adiv : signed divide
; ldiv : unsigned divide
; amod : signed modulo
; lmod : unsigned modulo
; shal : shift left arithmetic (_should_ be signed)
; shar : shift right arithmetic (_should_ be signed)
; shll : shift left logical (unsigned)
; shlr : shift right logical (unsigned)
;
; ** 32-bit routines: (not complete)
; allsh: shift left arithmetic (_should_ be signed)
; alrsh: shift right arithmetic (_should_ be signed)
; lllsh: shift left logical (unsigned)
; llrsh: shift right logical (unsigned)
; asallsh: shift left arithmetic (assign value, should be signed)
; asalrsh: shift right arithmetic (assign value, should be signed)
; aland: AND arithmetic (signed)
; lland: AND logical (unsigned)

; clever way to make sure usgard.h gets included
% main
#include "inc\\usgard.h"

%

; *** compare routines ***

% wrelop
; Compare word (hl <-> de)
wrelop:
  xor a
  sbc hl,de
  ret

% brelop
; Compare byte (??)
brelop:
  cp b
  ret

% amul lmul

; *** 16-bit arithmetic routines ***

; Multiply routine for amul and lmul
; a*de -> a hl (24-bit!)
mpy8__:
        ld bc,$0800
        ld h,c
        ld l,c
mpy8_1:
        add hl,hl
        rla
        jr nc,mpy8_2
        add hl,de
        adc a,c
mpy8_2:
        djnz mpy8_1
        ret

%amul lmul

; de*hl -> hl
; (same routine works for both signed and unsigned stuff)
amul:
lmul:
  ld a,h
  push hl
  call mpy8__ ; de*a -> a hl
  ld h,l    ; (a hl<<8) -> hl
  ld l,0
  pop bc

  ld a,c
  push hl
  call mpy8__
  pop de
  add hl,de
  ret

% adiv ldiv amod lmod

; Divide routine for adiv,ldiv,amod and lmod
; de/bc -> hl
; de%bc -> de
div16__:
        xor a
        ld h,a
        ld l,a
        ld a,16
div16_0:
        rl e
        rl d
        adc hl,hl
        sbc hl,bc
        jr nc, div16_1
        add hl,bc
div16_1:
        ccf
        dec a
        jr nz,div16_0
        ex de,hl
        adc hl,hl
        ret
% adiv

; hl/de -> hl
adiv:
  ld a,0
  bit 7,h
  jr z, adiv_l0
  
  push hl
  pop bc
  call NEG_BC ; NEG_BC returns -BC also in HL
  ld a,1

adiv_l0:
  bit 7,d
  jr z, adiv_l1
  
  push af
  push hl
  push de
  pop bc
  call NEG_BC
  push bc
  pop de
  pop hl
  pop af
  inc a

adiv_l1:

  push de
  pop bc
  push hl
  pop de
  push af
  call div16__  ; de/bc -> hl
  pop af
  cp 1
  ret nz
  
  push hl
  pop bc
  call NEG_BC ; NEG_BC returns -BC also in HL
  ret

% ldiv

; hl/de -> hl
ldiv:
;  ex de,hl
  push de
  pop bc
  push hl
  pop de
  call div16__  ; de/bc -> hl
  ret

% amod

; hl%de -> hl
amod:
  ld a,0
  bit 7,h
  jr z, amod_l0
  
  push hl
  pop bc
  call NEG_BC ; NEG_BC returns -BC also in HL
  ld a,1

amod_l0:
  bit 7,d
  jr z, amod_l1
  
  push af
  push hl
  push de
  pop bc
  call NEG_BC
  push bc
  pop de
  pop hl
  pop af
  inc a

amod_l1:

  push de
  pop bc
  push hl
  pop de
  push af
  call div16__  ; de%bc -> de
  ex de,hl
  pop af
  cp 1
  ret nz
  
  push hl
  pop bc
  call NEG_BC ; NEG_BC returns -BC also in HL
  ret

% lmod 
; hl%de -> hl
lmod:
  ex de,hl
  push hl
  pop bc
  call div16__  ; de/bc -> hl
  ex de,hl
  ret

% shal shll
shll:
; hl<<b ->hl
shal:
  ld a,b
  cp 0
  ret z
shal_l0:
  sla l
  rl h
  djnz shal_l0
  ret

% shar shlr
; hl>>b ->hl
shar:
  ld a,b
  cp 0
  ret z
shar_l0:
  sra h
  rr l
  djnz shar_l0
  ret

% alrsh llrsh 

; *** 32-bit arithmetic routines (beware, not complete!) ***
; hl de >> b -> hl de
llrsh:

alrsh:
  sra h
  rr l
  rr d
  rr e
  djnz alrsh
  ret

% allsh lllsh

; hl de << b -> hl de
lllsh:

allsh:
  sra e
  rr d
  rr l
  rr h
  djnz alrsh
  ret

% asallsh 
; dword (hl) << b ->dword (hl)

asallsh:
  push hl
  ld e,(hl)
  inc hl
  ld d,(hl)
  inc hl
  call LD_HL_MHL
  call allsh
  push hl
  pop bc
  pop hl
  ld (hl),e
  inc hl
  ld (hl),d
  inc hl
  ld (hl),c
  inc hl
  ld (hl),b
  inc hl
  ret

% asalrsh

; dword (hl) >> b ->dword (hl)
asalrsh:
  push hl
  ld e,(hl)
  inc hl
  ld d,(hl)
  inc hl
  call LD_HL_MHL
  call alrsh
  push hl
  pop bc
  pop hl
  ld (hl),e
  inc hl
  ld (hl),d
  inc hl
  ld (hl),c
  inc hl
  ld (hl),b
  inc hl
  ret

% aland lland

; dword (sp-4) & de hl -> de hl
aland:

lland:
  pop bc
  ld a,b
  and d
  ld d,a
  ld a,c
  and e
  ld e,a
  
  pop bc
  ld a,b
  and h
  ld h,a
  ld a,c
  and l
  ld l,a

  ret
